@@ -150,7 +150,7 @@ function twentyfifteen_categorized_blog() {
 		set_transient( 'twentyfifteen_categories', $all_the_cool_cats );
 	}
 
-	if ( $all_the_cool_cats > 1 ) {
+	if ( $all_the_cool_cats > 1 || is_preview() ) {
 		// This blog has more than 1 category so twentyfifteen_categorized_blog should return true.
 		return true;
 	} else {
@@ -241,3 +241,18 @@ function twentyfifteen_excerpt_more( $more ) {
 }
 add_filter( 'excerpt_more', 'twentyfifteen_excerpt_more' );
 endif;
+
+if ( ! function_exists( 'twentyfifteen_the_custom_logo' ) ) :
+/**
+ * Displays the optional custom logo.
+ *
+ * Does nothing if the custom logo is not available.
+ *
+ * @since Twenty Fifteen 1.5
+ */
+function twentyfifteen_the_custom_logo() {
+	if ( function_exists( 'the_custom_logo' ) ) {
+		the_custom_logo();
+	}
+}
+endif;
