@@ -37,12 +37,18 @@ class WP_Customize_Control {
 	public $instance_number;
 
 	/**
+	 * Customizer manager.
+	 *
+	 * @since 3.4.0
 	 * @access public
 	 * @var WP_Customize_Manager
 	 */
 	public $manager;
 
 	/**
+	 * Control ID.
+	 *
+	 * @since 3.4.0
 	 * @access public
 	 * @var string
 	 */
@@ -51,6 +57,7 @@ class WP_Customize_Control {
 	/**
 	 * All settings tied to the control.
 	 *
+	 * @since 3.4.0
 	 * @access public
 	 * @var array
 	 */
@@ -59,57 +66,101 @@ class WP_Customize_Control {
 	/**
 	 * The primary setting for the control (if there is one).
 	 *
+	 * @since 3.4.0
 	 * @access public
 	 * @var string
 	 */
 	public $setting = 'default';
 
 	/**
+	 * Capability required to use this control.
+	 *
+	 * Normally this is empty and the capability is derived from the capabilities
+	 * of the associated `$settings`.
+	 *
+	 * @since 4.5.0
+	 * @access public
+	 * @var string
+	 */
+	public $capability;
+
+	/**
+	 * Order priority to load the control in Customizer.
+	 *
+	 * @since 3.4.0
 	 * @access public
 	 * @var int
 	 */
 	public $priority = 10;
 
 	/**
+	 * Section the control belongs to.
+	 *
+	 * @since 3.4.0
 	 * @access public
 	 * @var string
 	 */
 	public $section = '';
 
 	/**
+	 * Label for the control.
+	 *
+	 * @since 3.4.0
 	 * @access public
 	 * @var string
 	 */
 	public $label = '';
 
 	/**
+	 * Description for the control.
+	 *
+	 * @since 4.0.0
 	 * @access public
 	 * @var string
 	 */
 	public $description = '';
 
 	/**
-	 * @todo: Remove choices
+	 * List of choices for 'radio' or 'select' type controls, where values are the keys, and labels are the values.
 	 *
+	 * @since 3.4.0
 	 * @access public
 	 * @var array
 	 */
 	public $choices = array();
 
 	/**
+	 * List of custom input attributes for control output, where attribute names are the keys and values are the values.
+	 *
+	 * Not used for 'checkbox', 'radio', 'select', 'textarea', or 'dropdown-pages' control types.
+	 *
+	 * @since 4.0.0
 	 * @access public
 	 * @var array
 	 */
 	public $input_attrs = array();
 
+	/**
+	 * Show UI for adding new content, currently only used for the dropdown-pages control.
+	 *
+	 * @since 4.7.0
+	 * @access public
+	 * @var bool
+	 */
+	public $allow_addition = false;
+
 	/**
 	 * @deprecated It is better to just call the json() method
+	 * @since 3.4.0
 	 * @access public
 	 * @var array
 	 */
 	public $json = array();
 
 	/**
+	 * Control's Type.
+	 *
+	 * @since 3.4.0
 	 * @access public
 	 * @var string
 	 */
@@ -133,15 +184,42 @@ class WP_Customize_Control {
 	/**
 	 * Constructor.
 	 *
-	 * Supplied $args override class property defaults.
+	 * Supplied `$args` override class property defaults.
 	 *
-	 * If $args['settings'] is not defined, use the $id as the setting ID.
+	 * If `$args['settings']` is not defined, use the $id as the setting ID.
 	 *
 	 * @since 3.4.0
 	 *
 	 * @param WP_Customize_Manager $manager Customizer bootstrap instance.
 	 * @param string               $id      Control ID.
-	 * @param array                $args    Optional. Arguments to override class property defaults.
+	 * @param array                $args    {
+	 *     Optional. Arguments to override class property defaults.
+	 *
+	 *     @type int                  $instance_number Order in which this instance was created in relation
+	 *                                                 to other instances.
+	 *     @type WP_Customize_Manager $manager         Customizer bootstrap instance.
+	 *     @type string               $id              Control ID.
+	 *     @type array                $settings        All settings tied to the control. If undefined, `$id` will
+	 *                                                 be used.
+	 *     @type string               $setting         The primary setting for the control (if there is one).
+	 *                                                 Default 'default'.
+	 *     @type int                  $priority        Order priority to load the control. Default 10.
+	 *     @type string               $section         Section the control belongs to. Default empty.
+	 *     @type string               $label           Label for the control. Default empty.
+	 *     @type string               $description     Description for the control. Default empty.
+	 *     @type array                $choices         List of choices for 'radio' or 'select' type controls, where
+	 *                                                 values are the keys, and labels are the values.
+	 *                                                 Default empty array.
+	 *     @type array                $input_attrs     List of custom input attributes for control output, where
+	 *                                                 attribute names are the keys and values are the values. Not
+	 *                                                 used for 'checkbox', 'radio', 'select', 'textarea', or
+	 *                                                 'dropdown-pages' control types. Default empty array.
+	 *     @type array                $json            Deprecated. Use WP_Customize_Control::json() instead.
+	 *     @type string               $type            Control type. Core controls include 'text', 'checkbox',
+	 *                                                 'textarea', 'radio', 'select', and 'dropdown-pages'. Additional
+	 *                                                 input types such as 'email', 'url', 'number', 'hidden', and
+	 *                                                 'date' are supported implicitly. Default 'text'.
+	 * }
 	 */
 	public function __construct( $manager, $id, $args = array() ) {
 		$keys = array_keys( get_object_vars( $this ) );
@@ -160,7 +238,7 @@ class WP_Customize_Control {
 		$this->instance_number = self::$instance_count;
 
 		// Process settings.
-		if ( empty( $this->settings ) ) {
+		if ( ! isset( $this->settings ) ) {
 			$this->settings = $id;
 		}
 
@@ -169,7 +247,7 @@ class WP_Customize_Control {
 			foreach ( $this->settings as $key => $setting ) {
 				$settings[ $key ] = $this->manager->get_setting( $setting );
 			}
-		} else {
+		} else if ( is_string( $this->settings ) ) {
 			$this->setting = $this->manager->get_setting( $this->settings );
 			$settings['default'] = $this->setting;
 		}
@@ -187,7 +265,6 @@ class WP_Customize_Control {
 	 * Check whether control is active to current Customizer preview.
 	 *
 	 * @since 4.0.0
-	 * @access public
 	 *
 	 * @return bool Whether the control is active to the current preview.
 	 */
@@ -196,7 +273,7 @@ class WP_Customize_Control {
 		$active = call_user_func( $this->active_callback, $this );
 
 		/**
-		 * Filter response of WP_Customize_Control::active().
+		 * Filters response of WP_Customize_Control::active().
 		 *
 		 * @since 4.0.0
 		 *
@@ -215,7 +292,6 @@ class WP_Customize_Control {
 	 * provide an 'active_callback' argument to the constructor.
 	 *
 	 * @since 4.0.0
-	 * @access public
 	 *
 	 * @return true Always true.
 	 */
@@ -257,6 +333,10 @@ class WP_Customize_Control {
 		$this->json['label'] = $this->label;
 		$this->json['description'] = $this->description;
 		$this->json['instanceNumber'] = $this->instance_number;
+
+		if ( 'dropdown-pages' === $this->type ) {
+			$this->json['allow_addition'] = $this->allow_addition;
+		}
 	}
 
 	/**
@@ -272,21 +352,32 @@ class WP_Customize_Control {
 	}
 
 	/**
-	 * Check if the theme supports the control and check user capabilities.
+	 * Checks if the user can use this control.
+	 *
+	 * Returns false if the user cannot manipulate one of the associated settings,
+	 * or if one of the associated settings does not exist. Also returns false if
+	 * the associated section does not exist or if its capability check returns
+	 * false.
 	 *
 	 * @since 3.4.0
 	 *
 	 * @return bool False if theme doesn't support the control or user doesn't have the required permissions, otherwise true.
 	 */
 	final public function check_capabilities() {
+		if ( ! empty( $this->capability ) && ! current_user_can( $this->capability ) ) {
+			return false;
+		}
+
 		foreach ( $this->settings as $setting ) {
-			if ( ! $setting->check_capabilities() )
+			if ( ! $setting || ! $setting->check_capabilities() ) {
 				return false;
+			}
 		}
 
 		$section = $this->manager->get_section( $this->section );
-		if ( isset( $section ) && ! $section->check_capabilities() )
+		if ( isset( $section ) && ! $section->check_capabilities() ) {
 			return false;
+		}
 
 		return true;
 	}
@@ -331,9 +422,9 @@ class WP_Customize_Control {
 		 *
 		 * @since 3.4.0
 		 *
-		 * @param WP_Customize_Control $this {@see WP_Customize_Control} instance.
+		 * @param WP_Customize_Control $this WP_Customize_Control instance.
 		 */
-		do_action( 'customize_render_control_' . $this->id, $this );
+		do_action( "customize_render_control_{$this->id}", $this );
 
 		$this->render();
 	}
@@ -383,7 +474,6 @@ class WP_Customize_Control {
 	 * Render the custom attributes for the control's input element.
 	 *
 	 * @since 4.0.0
-	 * @access public
 	 */
 	public function input_attrs() {
 		foreach ( $this->input_attrs as $attr => $value ) {
@@ -394,12 +484,12 @@ class WP_Customize_Control {
 	/**
 	 * Render the control's content.
 	 *
-	 * Allows the content to be overriden without having to rewrite the wrapper in $this->render().
+	 * Allows the content to be overridden without having to rewrite the wrapper in `$this::render()`.
 	 *
 	 * Supports basic input types `text`, `checkbox`, `textarea`, `radio`, `select` and `dropdown-pages`.
 	 * Additional input types such as `email`, `url`, `number`, `hidden` and `date` are supported implicitly.
 	 *
-	 * Control content can alternately be rendered in JS. See {@see WP_Customize_Control::print_template()}.
+	 * Control content can alternately be rendered in JS. See WP_Customize_Control::print_template().
 	 *
 	 * @since 3.4.0
 	 */
@@ -469,7 +559,7 @@ class WP_Customize_Control {
 					if ( ! empty( $this->description ) ) : ?>
 						<span class="description customize-control-description"><?php echo $this->description; ?></span>
 					<?php endif; ?>
-					<textarea rows="5" <?php $this->link(); ?>><?php echo esc_textarea( $this->value() ); ?></textarea>
+					<textarea rows="5" <?php $this->input_attrs(); ?> <?php $this->link(); ?>><?php echo esc_textarea( $this->value() ); ?></textarea>
 				</label>
 				<?php
 				break;
@@ -483,22 +573,58 @@ class WP_Customize_Control {
 					<span class="description customize-control-description"><?php echo $this->description; ?></span>
 				<?php endif; ?>
 
-				<?php $dropdown = wp_dropdown_pages(
+				<?php
+				$dropdown_name = '_customize-dropdown-pages-' . $this->id;
+				$show_option_none = __( '&mdash; Select &mdash;' );
+				$option_none_value = '0';
+				$dropdown = wp_dropdown_pages(
 					array(
-						'name'              => '_customize-dropdown-pages-' . $this->id,
+						'name'              => $dropdown_name,
 						'echo'              => 0,
-						'show_option_none'  => __( '&mdash; Select &mdash;' ),
-						'option_none_value' => '0',
+						'show_option_none'  => $show_option_none,
+						'option_none_value' => $option_none_value,
 						'selected'          => $this->value(),
 					)
 				);
+				if ( empty( $dropdown ) ) {
+					$dropdown = sprintf( '<select id="%1$s" name="%1$s">', esc_attr( $dropdown_name ) );
+					$dropdown .= sprintf( '<option value="%1$s">%2$s</option>', esc_attr( $option_none_value ), esc_html( $show_option_none ) );
+					$dropdown .= '</select>';
+				}
 
 				// Hackily add in the data link parameter.
 				$dropdown = str_replace( '<select', '<select ' . $this->get_link(), $dropdown );
+
+				// Even more hacikly add auto-draft page stubs.
+				// @todo Eventually this should be removed in favor of the pages being injected into the underlying get_pages() call. See <https://github.com/xwp/wp-customize-posts/pull/250>.
+				$nav_menus_created_posts_setting = $this->manager->get_setting( 'nav_menus_created_posts' );
+				if ( $nav_menus_created_posts_setting && current_user_can( 'publish_pages' ) ) {
+					$auto_draft_page_options = '';
+					foreach ( $nav_menus_created_posts_setting->value() as $auto_draft_page_id ) {
+						$post = get_post( $auto_draft_page_id );
+						if ( $post && 'page' === $post->post_type ) {
+							$auto_draft_page_options .= sprintf( '<option value="%1$s">%2$s</option>', esc_attr( $post->ID ), esc_html( $post->post_title ) );
+						}
+					}
+					if ( $auto_draft_page_options ) {
+						$dropdown = str_replace( '</select>', $auto_draft_page_options . '</select>', $dropdown );
+					}
+				}
+
 				echo $dropdown;
 				?>
 				</label>
-				<?php
+				<?php if ( $this->allow_addition && current_user_can( 'publish_pages' ) && current_user_can( 'edit_theme_options' ) ) : // Currently tied to menus functionality. ?>
+					<button type="button" class="button-link add-new-toggle"><?php
+						/* translators: %s: add new page label */
+						printf( __( '+ %s' ), get_post_type_object( 'page' )->labels->add_new_item );
+					?></button>
+					<div class="new-content-item">
+						<label for="create-input-<?php echo $this->id; ?>"><span class="screen-reader-text"><?php _e( 'New page title' ); ?></span></label>
+						<input type="text" id="create-input-<?php echo $this->id; ?>" class="create-item-input" placeholder="<?php esc_attr_e( 'New page title&hellip;' ); ?>">
+						<button type="button" class="button add-content"><?php _e( 'Add' ); ?></button>
+					</div>
+				<?php endif;
 				break;
 			default:
 				?>
@@ -520,7 +646,7 @@ class WP_Customize_Control {
 	 * Render the control's JS template.
 	 *
 	 * This function is only run for control types that have been registered with
-	 * {@see WP_Customize_Manager::register_control_type()}.
+	 * WP_Customize_Manager::register_control_type().
 	 *
 	 * In the future, this will also print the template for the control's container
 	 * element and be override-able.
@@ -539,7 +665,7 @@ class WP_Customize_Control {
 	 * An Underscore (JS) template for this control's content (but not its container).
 	 *
 	 * Class variables for this control class are available in the `data` JS object;
-	 * export custom variables by overriding {@see WP_Customize_Control::to_json()}.
+	 * export custom variables by overriding WP_Customize_Control::to_json().
 	 *
 	 * @see WP_Customize_Control::print_template()
 	 *
@@ -549,53 +675,92 @@ class WP_Customize_Control {
 
 }
 
-/** WP_Customize_Color_Control class */
+/**
+ * WP_Customize_Color_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-color-control.php' );
 
-/** WP_Customize_Media_Control class */
+/**
+ * WP_Customize_Media_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-media-control.php' );
 
-/** WP_Customize_Upload_Control class */
+/**
+ * WP_Customize_Upload_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-upload-control.php' );
 
-/** WP_Customize_Image_Control class */
+/**
+ * WP_Customize_Image_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-image-control.php' );
 
-/** WP_Customize_Background_Image_Control class */
+/**
+ * WP_Customize_Background_Image_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-background-image-control.php' );
 
-/** WP_Customize_Cropped_Image_Control class */
+/**
+ * WP_Customize_Background_Position_Control class.
+ */
+require_once( ABSPATH . WPINC . '/customize/class-wp-customize-background-position-control.php' );
+
+/**
+ * WP_Customize_Cropped_Image_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-cropped-image-control.php' );
 
-/** WP_Customize_Site_Icon_Control class */
+/**
+ * WP_Customize_Site_Icon_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-site-icon-control.php' );
 
-/** WP_Customize_Header_Image_Control class */
+/**
+ * WP_Customize_Header_Image_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-header-image-control.php' );
 
-/** WP_Customize_Theme_Control class */
+/**
+ * WP_Customize_Theme_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-theme-control.php' );
 
-/** WP_Widget_Area_Customize_Control class */
+/**
+ * WP_Widget_Area_Customize_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-widget-area-customize-control.php' );
 
-/** WP_Widget_Form_Customize_Control class */
+/**
+ * WP_Widget_Form_Customize_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-widget-form-customize-control.php' );
 
-/** WP_Customize_Nav_Menu_Control class */
+/**
+ * WP_Customize_Nav_Menu_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-nav-menu-control.php' );
 
-/** WP_Customize_Nav_Menu_Item_Control class */
+/**
+ * WP_Customize_Nav_Menu_Item_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-nav-menu-item-control.php' );
 
-/** WP_Customize_Nav_Menu_Location_Control class */
+/**
+ * WP_Customize_Nav_Menu_Location_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-nav-menu-location-control.php' );
 
-/** WP_Customize_Nav_Menu_Name_Control class */
+/**
+ * WP_Customize_Nav_Menu_Name_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-nav-menu-name-control.php' );
 
-/** WP_Customize_Nav_Menu_Auto_Add_Control class */
+/**
+ * WP_Customize_Nav_Menu_Auto_Add_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-nav-menu-auto-add-control.php' );
 
-/** WP_Customize_New_Menu_Control class */
+/**
+ * WP_Customize_New_Menu_Control class.
+ */
 require_once( ABSPATH . WPINC . '/customize/class-wp-customize-new-menu-control.php' );
