@@ -35,9 +35,23 @@ final class WP_Customize_Widgets {
 	 * @var array
 	 */
 	protected $core_widget_id_bases = array(
-		'archives', 'calendar', 'categories', 'links', 'meta',
-		'nav_menu', 'pages', 'recent-comments', 'recent-posts',
-		'rss', 'search', 'tag_cloud', 'text',
+		'archives',
+		'calendar',
+		'categories',
+		'custom_html',
+		'links',
+		'media_audio',
+		'media_image',
+		'media_video',
+		'meta',
+		'nav_menu',
+		'pages',
+		'recent-comments',
+		'recent-posts',
+		'rss',
+		'search',
+		'tag_cloud',
+		'text',
 	);
 
 	/**
@@ -721,6 +735,47 @@ final class WP_Customize_Widgets {
 			</div>'
 		);
 
+		/*
+		 * Gather all strings in PHP that may be needed by JS on the client.
+		 * Once JS i18n is implemented (in #20491), this can be removed.
+		 */
+		$some_non_rendered_areas_messages = array();
+		$some_non_rendered_areas_messages[1] = html_entity_decode(
+			/* translators: placeholder is the number of other widget areas registered but not rendered */
+			__( 'Your theme has 1 other widget area, but this particular page doesn&#8217;t display it.' ),
+			ENT_QUOTES,
+			get_bloginfo( 'charset' )
+		);
+		$registered_sidebar_count = count( $wp_registered_sidebars );
+		for ( $non_rendered_count = 2; $non_rendered_count < $registered_sidebar_count; $non_rendered_count++ ) {
+			$some_non_rendered_areas_messages[ $non_rendered_count ] = html_entity_decode( sprintf(
+				/* translators: placeholder is the number of other widget areas registered but not rendered */
+				_n(
+					'Your theme has %s other widget area, but this particular page doesn&#8217;t display it.',
+					'Your theme has %s other widget areas, but this particular page doesn&#8217;t display them.',
+					$non_rendered_count
+				),
+				number_format_i18n( $non_rendered_count )
+			), ENT_QUOTES, get_bloginfo( 'charset' ) );
+		}
+
+		if ( 1 === $registered_sidebar_count ) {
+			$no_areas_shown_message = html_entity_decode( sprintf(
+				/* translators: placeholder is the total number of widget areas registered */
+				__( 'Your theme has 1 widget area, but this particular page doesn&#8217;t display it.' )
+			), ENT_QUOTES, get_bloginfo( 'charset' ) );
+		} else {
+			$no_areas_shown_message = html_entity_decode( sprintf(
+				/* translators: placeholder is the total number of widget areas registered */
+				_n(
+					'Your theme has %s widget area, but this particular page doesn&#8217;t display it.',
+					'Your theme has %s widget areas, but this particular page doesn&#8217;t display them.',
+					$registered_sidebar_count
+				),
+				number_format_i18n( $registered_sidebar_count )
+			), ENT_QUOTES, get_bloginfo( 'charset' ) );
+		}
+
 		$settings = array(
 			'registeredSidebars'   => array_values( $wp_registered_sidebars ),
 			'registeredWidgets'    => $wp_registered_widgets,
@@ -733,10 +788,13 @@ final class WP_Customize_Widgets {
 				'error'            => __( 'An error has occurred. Please reload the page and try again.' ),
 				'widgetMovedUp'    => __( 'Widget moved up' ),
 				'widgetMovedDown'  => __( 'Widget moved down' ),
-				'noAreasRendered'  => __( 'There are no widget areas on the page shown, however other pages in this theme do have them.' ),
+				'navigatePreview'  => __( 'You can navigate to other pages on your site while using the Customizer to view and edit the widgets displayed on those pages.' ),
+				'someAreasShown'   => $some_non_rendered_areas_messages,
+				'noAreasShown'     => $no_areas_shown_message,
 				'reorderModeOn'    => __( 'Reorder mode enabled' ),
 				'reorderModeOff'   => __( 'Reorder mode closed' ),
 				'reorderLabelOn'   => esc_attr__( 'Reorder widgets' ),
+				/* translators: placeholder is the count for the number of widgets found */
 				'widgetsFound'     => __( 'Number of widgets found: %d' ),
 				'noWidgetsFound'   => __( 'No widgets found.' ),
 			),
